@@ -90,6 +90,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 //
 //******************************************************************************
 
+extern int G_weaponscale;
+extern boolean iG_aimCross;
+
 boolean WriteSoundFile   = true;
 
 int     FXMode           = 0;
@@ -98,12 +101,19 @@ int     MusicMode        = 0;
 int     MUvolume         = 196;
 int     FXvolume         = 196;
 
+#ifdef DOS
 fx_blaster_config SBSettings =
    {
    0x220, fx_SB, 7, 1, 5, 0x330, 0x620
    };
+#endif
 
 boolean mouseenabled     = 1;
+boolean usemouselook     = 0;
+int     inverse_mouse    = 1; //set  to -1 to invert mouse
+boolean usejump          = 0;
+boolean sdl_fullscreen   = 1;
+
 boolean joystickenabled  = 0;
 boolean joypadenabled    = 0;
 int     joystickport     = 0;
@@ -112,7 +122,9 @@ int     threshold        = 1;
 int     NumVoices        = 4;
 int     NumChannels      = 1;
 int     NumBits          = 8;
+#ifdef DOS
 int     MidiAddress      = 0x330;
+#endif
 boolean cybermanenabled  = false;
 boolean assassinenabled  = false;
 boolean spaceballenabled = false;
@@ -288,14 +300,17 @@ boolean ParseSoundFile (void)
 
       ReadInt ("NumBits",&NumBits);
 
+#ifdef DOS
       // Read in Midi Address
 
       ReadInt ("MidiAddress",&MidiAddress);
+#endif
 
       // Read in stereo reversal
 
       ReadBoolean ("StereoReverse",&stereoreversed);
 
+#ifdef DOS
       // Read in Sound Blaster info
       ReadUnsigned ("SBType",  &SBSettings.Type );
       ReadUnsigned ("SBPort",  &SBSettings.Address );
@@ -304,6 +319,7 @@ boolean ParseSoundFile (void)
       ReadUnsigned ("SBDma16", &SBSettings.Dma16 );
       ReadUnsigned ("SBMidi",  &SBSettings.Midi );
       ReadUnsigned ("SBEmu",   &SBSettings.Emu );
+#endif
    }
    else
       retval = false;
@@ -324,7 +340,9 @@ void SetSoundDefaultValues
    )
 
    {
+#ifdef DOS
    fx_blaster_config blaster;
+#endif
    int status;
 
    //
@@ -337,7 +355,6 @@ void SetSoundDefaultValues
    NumVoices   = 8;
    NumChannels = 2;
    NumBits     = 16;
-   MidiAddress = 0x330;
    stereoreversed = false;
    #else
    MusicMode   = 0;
@@ -347,7 +364,6 @@ void SetSoundDefaultValues
    NumBits     = 8;
    MidiAddress = 0x330;
    stereoreversed = false;
-   #endif
 
    status = FX_GetBlasterSettings( &blaster );
    if ( status == FX_Ok )
@@ -360,6 +376,7 @@ void SetSoundDefaultValues
       SBSettings.Midi      = blaster.Midi;
       SBSettings.Emu       = blaster.Emu;
       }
+   #endif
    }
 
 
@@ -377,6 +394,7 @@ extern char    pword[ 13 ];
 void ConvertStringToPasswordString ( char * string )
 {
    int i;
+   unsigned int j;
    char temp[3];
 
    memset(temp,0,sizeof(temp));
@@ -384,7 +402,11 @@ void ConvertStringToPasswordString ( char * string )
    for (i=0;i<13;i++)
       {
       memcpy(&temp[0],&string[i<<1],2);
-      sscanf(&temp[0],"%x",(unsigned int *)&passwordstring[i]);
+      sscanf(&temp[0],"%x",&j);
+      passwordstring[i+0] = j & 0xff; j >>= 8;
+      passwordstring[i+1] = j & 0xff; j >>= 8;
+      passwordstring[i+2] = j & 0xff; j >>= 8;
+      passwordstring[i+3] = j & 0xff;
       }
 }
 
@@ -486,25 +508,53 @@ boolean ParseConfigFile (void)
    if (version == ROTTVERSION)
    {
       // Read in MouseEnabled
-
       ReadBoolean("MouseEnabled",&mouseenabled);
 
-      // Read in JoystickEnabled
+      // Read in UseMouseLook
+      ReadBoolean("UseMouseLook",&usemouselook);
 
+      ReadInt("InverseMouse",&inverse_mouse);
+
+	  // Read in UseJump
+      ReadBoolean("UseJump",&usejump);
+
+      // Read in CrossHair
+      ReadBoolean("CrossHair",&iG_aimCross);
+
+      // Read in JoystickEnabled
       ReadBoolean("JoystickEnabled",&joystickenabled);
 
       // Read in JoypadEnabled
-
       ReadBoolean("JoypadEnabled",&joypadenabled);
 
       // Read in JoystickPort
 
       ReadInt("JoystickPort",&joystickport);
+      
+      // Read in fullscreen
+      ReadBoolean("FullScreen", &sdl_fullscreen);
+      
+      // Read in resolution
+      ReadInt("ScreenWidth", &iGLOBAL_SCREENWIDTH);
+      ReadInt("ScreenHeight", &iGLOBAL_SCREENHEIGHT);
 
       // Read in ViewSize
 
       ReadInt("ViewSize",&viewsize);
 
+      // Read in Weaponscale
+
+      ReadInt("Weaponscale",&G_weaponscale);//bna added
+	   if ((G_weaponscale <150)||(G_weaponscale>600)){
+		   if (iGLOBAL_SCREENWIDTH == 320){
+				G_weaponscale=168;
+		   }else if (iGLOBAL_SCREENWIDTH == 640){
+				G_weaponscale=299;		
+		   }else if (iGLOBAL_SCREENWIDTH == 800) {
+				G_weaponscale=376;
+		   }	   
+	   }
+
       // Read in MouseAdjustment
 
       ReadInt("MouseAdjustment",&mouseadjustment);
@@ -1519,6 +1569,7 @@ void WriteSoundConfig
    SafeWriteString(file,"\n;\n");
    SafeWriteString(file,"; Music Modes\n");
    SafeWriteString(file,"; 0  -  Off\n");
+#ifdef DOS
    SafeWriteString(file,"; 1  -  UltraSound\n");
    SafeWriteString(file,"; 2  -  Sound Blaster\n");
    SafeWriteString(file,"; 3  -  Sound Man 16\n");
@@ -1529,6 +1580,9 @@ void WriteSoundConfig
    SafeWriteString(file,"; 8  -  General Midi\n");
    SafeWriteString(file,"; 9  -  Sound Canvas\n");
    SafeWriteString(file,"; 10 -  Adlib\n");
+#else
+   SafeWriteString(file,"; 6  -  On\n");
+#endif
    WriteParameter(file,"MusicMode        ",MusicMode);
 
    // Write out FX Mode
@@ -1536,6 +1590,7 @@ void WriteSoundConfig
    SafeWriteString(file,"\n;\n");
    SafeWriteString(file,"; FX Modes\n");
    SafeWriteString(file,"; 0  -  Off\n");
+#ifdef DOS
    SafeWriteString(file,"; 1  -  UltraSound\n");
    SafeWriteString(file,"; 2  -  Sound Blaster\n");
    SafeWriteString(file,"; 3  -  Sound Man 16\n");
@@ -1546,6 +1601,9 @@ void WriteSoundConfig
    SafeWriteString(file,"; 8  -  Disney Sound Source\n");
    SafeWriteString(file,"; 9  -  Tandy Sound Source\n");
    SafeWriteString(file,"; 10 -  PC Speaker\n");
+#else
+   SafeWriteString(file,"; 6  -  On\n");
+#endif
    WriteParameter(file,"FXMode           ",FXMode);
 
    // Write in Music Volume
@@ -1585,6 +1643,7 @@ void WriteSoundConfig
    SafeWriteString(file,"; 16 bit\n");
    WriteParameter(file,"NumBits          ",NumBits);
 
+#ifdef DOS
    // Write out Midi Address
 
    SafeWriteString(file,"\n;\n");
@@ -1599,6 +1658,7 @@ void WriteSoundConfig
    SafeWriteString(file,"; $370\n");
    SafeWriteString(file,"; $380\n");
    WriteParameterHex(file,"MidiAddress      ",MidiAddress);
+#endif
 
    // Write out stereo reversal
 
@@ -1608,6 +1668,7 @@ void WriteSoundConfig
    SafeWriteString(file,"; 1 reverse stereo\n");
    WriteParameter (file,"StereoReverse      ",stereoreversed);
 
+#ifdef DOS
    // Write out Sound Blaster info
 
    SafeWriteString(file,"\n;\n");
@@ -1619,6 +1680,7 @@ void WriteSoundConfig
    WriteParameter(file, "SBDma16          ", SBSettings.Dma16 );
    WriteParameterHex(file, "SBMidi           ", SBSettings.Midi );
    WriteParameterHex(file, "SBEmu            ", SBSettings.Emu );
+#endif
 
    close (file);
    }
@@ -1678,6 +1740,30 @@ void WriteConfig (void)
    SafeWriteString(file,"; 0 - Mouse Disabled\n");
    WriteParameter(file,"MouseEnabled     ",mouseenabled);
 
+   // Write out UseMouseLook
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; 1 - UseMouseLook Enabled\n");
+   SafeWriteString(file,"; 0 - UseMouseLook Disabled\n");
+   WriteParameter(file,"UseMouseLook     ",usemouselook);
+
+   // Write out InverseMouse
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; 1 - Normal Mouse Enabled\n");
+   SafeWriteString(file,"; -1 - Inverse Mouse Enabled\n");
+   WriteParameter(file,"InverseMouse     ",inverse_mouse);
+
+   // Write out UseJump
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; 1 - usejump Enabled\n");
+   SafeWriteString(file,"; 0 - usejump Disabled\n");
+   WriteParameter(file,"UseJump          ",usejump);
+
+   // Write out CrossHair
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; 1 - CrossHair Enabled\n");
+   SafeWriteString(file,"; 0 - CrossHair Disabled\n");
+   WriteParameter(file,"CrossHair        ", iG_aimCross);
+
    // Write out JoystickEnabled
 
    SafeWriteString(file,"\n;\n");
@@ -1699,6 +1785,19 @@ void WriteConfig (void)
    SafeWriteString(file,"; 1 - Use Joystick Port 2\n");
    WriteParameter(file,"JoystickPort     ",joystickport);
 
+   // Write out fullscreen
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; 0 - Start in windowed mode\n");
+   SafeWriteString(file,"; 1 - Start in fullscreen mode\n");
+   WriteParameter(file,"FullScreen       ",sdl_fullscreen);
+      
+   // Write out resolution
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; Screen Resolution, supported resolutions: \n");
+   SafeWriteString(file,"; 320x200, 640x480 and 800x600\n");
+   WriteParameter(file,"ScreenWidth      ",iGLOBAL_SCREENWIDTH);
+   WriteParameter(file,"ScreenHeight     ",iGLOBAL_SCREENHEIGHT);
+
    // Write out ViewSize
 
    SafeWriteString(file,"\n;\n");
@@ -1706,6 +1805,25 @@ void WriteConfig (void)
    SafeWriteString(file,"; (smallest) 0 - 10 (largest)\n");
    WriteParameter(file,"ViewSize         ",viewsize);
 
+   // Write out WEAPONSCALE  bna added
+
+   SafeWriteString(file,"\n;\n");
+   SafeWriteString(file,"; Size of Weaponscale.\n");
+   SafeWriteString(file,"; (smallest) 150 - 600 (largest)\n");
+   G_weaponscale = (weaponscale * 168 )/65536;
+
+   if ((G_weaponscale <150)||(G_weaponscale>600)){
+	   if (iGLOBAL_SCREENWIDTH == 320){
+			G_weaponscale=168;
+	   }else if (iGLOBAL_SCREENWIDTH == 640){
+			G_weaponscale=299;		
+	   }else if (iGLOBAL_SCREENWIDTH == 800) {
+			G_weaponscale=376;
+	   }	   
+   }
+   WriteParameter(file,"Weaponscale         ",G_weaponscale);
+
+
    // Write out MouseAdjustment
 
    SafeWriteString(file,"\n;\n");
